/**
 * Copyright (c) 2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.admin.jmx.registry;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;

import org.ow2.petals.admin.api.exception.RegistryRegexpPatternException;
import org.ow2.petals.admin.registry.Endpoint;
import org.ow2.petals.admin.registry.Endpoint.EndpointType;
import org.ow2.petals.admin.registry.RegistryView;
import org.ow2.petals.jmx.commons.PetalsEndpointRegistryServiceConstants;

/**
 * 
 * @author Nicolas Oddoux - EBM WebSourcing
 */
public class JMXRegistryView implements RegistryView {

    private final List<Endpoint> endpoints;
    
    private final Map<String, List<Endpoint>> endpointsByServiceName;
    
    private final Map<String, List<Endpoint>> endpointsByInterfaceName;
    
    public JMXRegistryView(List<Map<String, Object>> endpointMaps, String endpointNameRegex,
            String serviceNameRegex, String interfaceNameRegex)
            throws RegistryRegexpPatternException {
        this.endpoints = new ArrayList<Endpoint>();
        this.endpointsByServiceName = new HashMap<String, List<Endpoint>>();
        this.endpointsByInterfaceName = new HashMap<String, List<Endpoint>>();
        
        initialize(endpointMaps, endpointNameRegex, serviceNameRegex, interfaceNameRegex);
    }

    private void initialize(List<Map<String, Object>> endpointMaps, String endpointNameRegex,
            String serviceNameRegex, String interfaceNameRegex)
            throws RegistryRegexpPatternException {
        try {
            Pattern endpointNamePattern = Pattern.compile(endpointNameRegex);
            Pattern serviceNamePattern = Pattern.compile(serviceNameRegex);
            Pattern interfaceNamePattern = Pattern.compile(interfaceNameRegex);
            
            for (Map<String, Object> endpointMap : endpointMaps) {
                String endpointName = (String) endpointMap
                        .get(PetalsEndpointRegistryServiceConstants.Endpoint.ENDPOINT_NAME);
                String endpointTypeStr = (String) endpointMap
                        .get(PetalsEndpointRegistryServiceConstants.Endpoint.TYPE);
                EndpointType endpointType;
                if (endpointTypeStr
                        .equals(PetalsEndpointRegistryServiceConstants.EndpointType.INTERNAL)) {
                    endpointType = EndpointType.INTERNAL;
                } else {
                    endpointType = EndpointType.EXTERNAL;
                }
                String containerName = (String) endpointMap
                        .get(PetalsEndpointRegistryServiceConstants.Endpoint.CONTAINER_NAME);
                String componentName = (String) endpointMap
                        .get(PetalsEndpointRegistryServiceConstants.Endpoint.COMPONENT_NAME);
                String serviceName = (String) endpointMap
                        .get(PetalsEndpointRegistryServiceConstants.Endpoint.SERVICE_NAME);
                List<String> interfaceNames = Arrays.asList((String[]) endpointMap
                        .get(PetalsEndpointRegistryServiceConstants.Endpoint.INTERFACE_NAMES));

                if (isSelectedEndpoint(endpointName, serviceName, interfaceNames,
                        endpointNamePattern, serviceNamePattern, interfaceNamePattern)) {
                    Endpoint endpoint = new Endpoint(endpointName, endpointType, containerName,
                            componentName, serviceName, interfaceNames);
                    this.endpoints.add(endpoint);

                    List<Endpoint> endpointsForThisServiceName;
                    if (this.endpointsByServiceName.containsKey(serviceName)) {
                        endpointsForThisServiceName = this.endpointsByServiceName.get(serviceName);
                    } else {
                        endpointsForThisServiceName = new ArrayList<Endpoint>();
                        this.endpointsByServiceName.put(serviceName, endpointsForThisServiceName);
                    }
                    endpointsForThisServiceName.add(endpoint);

                    for (String interfaceName : interfaceNames) {
                        List<Endpoint> endpointsForThisInterfaceName;
                        if (this.endpointsByInterfaceName.containsKey(interfaceName)) {
                            endpointsForThisInterfaceName = this.endpointsByInterfaceName
                                    .get(interfaceName);
                        } else {
                            endpointsForThisInterfaceName = new ArrayList<Endpoint>();
                            this.endpointsByInterfaceName.put(interfaceName,
                                    endpointsForThisInterfaceName);
                        }
                        endpointsForThisInterfaceName.add(endpoint);
                    }
                }
            }
        } catch (final PatternSyntaxException e) {
            throw new RegistryRegexpPatternException(e.getPattern(), e.getDescription());
        }
    }
    
    private static final boolean isSelectedEndpoint(String endpointName, String serviceName,
            List<String> interfaceNames, Pattern endpointNamePattern, Pattern serviceNamePattern,
            Pattern interfaceNamePattern) {
        
        Matcher endpointNameMatcher = endpointNamePattern.matcher(endpointName);
        boolean isSelectedEndpoint = endpointNameMatcher.matches();
        
        if(isSelectedEndpoint) {
            Matcher serviceNameMatcher = serviceNamePattern.matcher(serviceName);
            isSelectedEndpoint = serviceNameMatcher.matches();
            
            if(isSelectedEndpoint) {
                for (String interfaceName : interfaceNames) {
                    Matcher interfaceNameMatcher = interfaceNamePattern.matcher(interfaceName);
                    isSelectedEndpoint = interfaceNameMatcher.matches();
                    if(isSelectedEndpoint) {
                        break;
                    }
                }
            }
        }
        
        return isSelectedEndpoint;
    }

    @Override
    public List<Endpoint> getAllEndpoints() {
        return this.endpoints;
    }

    @Override
    public Map<String, List<Endpoint>> getListOfEndpointsByServiceName() {
        return this.endpointsByServiceName;
    }

    @Override
    public Map<String, List<Endpoint>> getListOfEndpointsByInterfaceName() {
        return this.endpointsByInterfaceName;
    }

}
